#!/usr/bin/env python3
import enum
import numpy as np
import urllib.request as req
import math
import collections
import sys
import readline
import curses

class Moves(enum.Enum):
	left = "L"
	right = "p"
	up = "N"
	down = "D"

class Player:
	def __init__(self, xpos, ypos, treasure, score):
		self.xpos = xpos
		self.ypos = ypos
		self.treasure = treasure
		self.score = score

class Server:
	GameEnd = collections.namedtuple("GameEnd", ["score", "escore"])

	class Response:
		def __init__(self):
			board = None
			player = None
			eplayer = None

	def __init__(self, url):
		self.url = url
		self.id = None
		self.moves_left = None
		
	def parse_splitin(self, splitin):
		response = Server.Response()

		response.board = Board("".join(splitin[2:12]))

		playerinfo = splitin[12].split()
		response.player = Player(int(playerinfo[1]) - 1,
			int(playerinfo[0]) - 1, int(playerinfo[2]),
			int(playerinfo[3]))
		if(len(splitin) > 14): # Hra s dvoumi hráči
			eplayerinfo = splitin[13].split()
			response.eplayer = Player(int(eplayerinfo[1]) - 1,
				int(eplayerinfo[0]) - 1, int(eplayerinfo[2]),
				int(eplayerinfo[3]))

		return response

	def start(self):
		resp = req.urlopen(self.url + "/start")
		text = resp.read().replace(b"\r\n", b"\n").decode()
		splitin = text.splitlines()
		self.id = int(splitin[0])
		self.moves_left = int(splitin[1])
		
		return self.parse_splitin(splitin)
	
	def submit_move(self, move):
		resp = req.urlopen(self.url + "/tah?id=" + str(self.id) + "&pohyb=" + move)
		text = resp.read().replace(b"\r\n", b"\n").decode()
		
		if text.startswith("Konec hry:"):
			splitin = text.split()
			if len(splitin) == 3: # Hra s jedním hráčem
				return Server.GameEnd(int(splitin[2]), None)
			else: #Hra dvou hráčů
				return Server.GameEnd(int(splitin[2]), int(splitin[3]))
		
		if text.startswith("Chyba:"):
			splitin = text.split()
			print("Došlo k chybě:", splitin[1])
			sys.exit(1)

		splitin = text.splitlines()
		self.moves_left = int(splitin[1])
		return self.parse_splitin(splitin)
	
class BPieces(enum.Enum):
	space = ".."
	wall = "XX"
	base = "++"
	ebase = "--"

class Board:
	def __init__(self, text):
		"""
		Inicializuje hrací pole z textu
		text by měl být string o 20*10 znaků (žádné newliny)
		"""
		self.board = np.zeros((10, 10), dtype=object)
		for i in range(10*10):
			fstr =  text[i*2:i*2 + 2]
			if fstr.isdigit():
				fval = int(fstr)
			else:
				fval = fstr

			self.set_piece(math.floor(i/10), i%10, fstr)
	def set_piece(self, x, y, piece):
		self.board[x][y] = piece
	
	def get_piece(self, x, y):
		return self.board[x][y]
	
	def get_treasures(self):
		treasures = [] #Format: [(value, x, y, distance), ...]

		i = 1
		for col in self.board:
			j = 0
			for field in col:
				if field.isdigit():
					treasures.append((int(field, i, j, math.sqrt(i**2 + j**2))))
				j += 1
			i += 1

		treasures.sort(key=lambda x: x[0]/x[3]) # Sort by distance and value
		return treasures
	
	def get_next_step_to(self, xfrom, yfrom, xto, yto):
		distances = np.zeros((10, 10))
		directions = [(1, 0), (-1, 0), (0, 1), (0, -1)]
		def bfsearch_dist(x, y):
			for d in directions:
				nx, ny = x + d[0], y + d[1]
				if not self.get_piece(nx, ny) == BPieces.wall.value and\
					not distances[nx, ny] <= distances[x, y]:
					distances[nx, ny] = distances[x, y] + 1
					bfsearch(nx, ny)
		
		route = []
		def bfsearch_route(x, y):
			min_dist = None
			min_dist_x = None
			min_dist_y = None
			for d in directions:
				nx, ny = x + d[0], y + d[1]
				if nx == xto and ny = yto:
					return
				if min_dist == None or distances[nx, ny] < min_dist:
					min_dist = distances[nx, ny]
					min_dist_x = nx
					min_dist_y = ny
			route.append((min_dist_x, min_dist_y))
			bfsearch_route(min_dist_x, min_dist_y)

		bfsearch_dist(xfrom, yfrom)
		bfsearch_route(xto, yto)
		return route[-1]

class Game:
	def __init__(self):
		if len(sys.argv) != 2:
			print("Syntaxe:", sys.argv[0], "server URL")
			print("napr.", sys.argv[0], "http://localhost:8000")
			sys.exit(1)
	
	def run(self):
		

game = Game()
game.run()

